package com.adc.da.znks.controller;

import com.adc.da.base.web.BaseController;
import com.adc.da.util.http.ResponseMessage;
import com.adc.da.util.http.Result;
import com.adc.da.znks.entity.ResponseEntity;
import com.adc.da.znks.entity.User;
import com.adc.da.znks.service.LockRecordService;
import com.adc.da.znks.service.UserService;
import com.adc.da.znks.util.GenResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

/**
 * @author David
 */
@RestController
@RequestMapping("/api/user")
@Api(description = "|User|")
public class UserController extends BaseController<User> {

    @Autowired
    private UserService userService;
    @Autowired
    private LockRecordService lockRecordService;

    private static final Logger log = LoggerFactory.getLogger(UserController.class);

    @ApiOperation(value = "|User|登录或者注册用户")
    @PostMapping(value = {"/loginOrRegisterUser"})
    public ResponseEntity<Boolean> loginOrRegisterUser(String phoneNumber ,String password) {
        boolean flag = userService.loginOrRegister(phoneNumber, password);
        if (flag) {
            return GenResponse.success(String.valueOf(HttpStatus.OK.value()), "登录或者注册用户成功", flag);
        } else {
            return GenResponse.fail("登录或者注册用户失败");
        }
    }

    @ApiOperation(value = "|User|查询当前手机号是否已经注册")
    @GetMapping(value = {"/isRegister"})
    public ResponseEntity<Boolean> isRegister(String phoneNumber) {
        boolean flag = userService.getUserByPhoneNumber(phoneNumber);
        return GenResponse.success(String.valueOf(HttpStatus.OK.value()), "登录或者注册用户成功", flag);
    }

    @ApiOperation(value = "|User|更新密码")
    @PostMapping("/updatePassword")
    public ResponseEntity<Boolean> update(String phoneNumber,String password) {
        User user = new User();
        user.setTemphone(phoneNumber);
        user.setPlainPassword(password);
        boolean flag = userService.updatePassword(user);
        if (flag) {
            return GenResponse.success(String.valueOf(HttpStatus.OK.value()), "更新密码成功", flag);
        } else {
            return GenResponse.fail("更新密码失败");
        }
    }

    @ApiOperation(value = "|User|更新个推ClientId")
    @PostMapping("/updateClientId")
    public ResponseEntity updateClientId(String clientId, String phoneNumber) {
        userService.updateClientIdByPhoneNumber(clientId, phoneNumber);
        return GenResponse.success();
    }
    //selectLockRecord

    @ApiOperation(value = "开锁日志")
    @GetMapping("/listLockRecord")
    public ResponseMessage listLockRecord(String phone,String number) {
        return Result.success(lockRecordService.selectLockRecord(phone,number));
    }

}
