package com.adc.da.znks.service;


import com.adc.da.base.service.BaseService;
import com.adc.da.util.utils.CollectionUtils;
import com.adc.da.util.utils.UUID;
import com.adc.da.znks.dao.UserDao;
import com.adc.da.znks.entity.User;
import com.adc.da.znks.page.UserPage;
import com.adc.da.znks.util.SecurityUtils;
import org.apache.http.util.Asserts;
import org.apache.ibatis.annotations.Param;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <br>
 * <b>功能：</b>UserService<br>
 *
 * @author ley
 */
@Service
@Transactional(value = "transactionManager", propagation = Propagation.REQUIRED, rollbackFor = Throwable.class)
public class UserService extends BaseService<User, String> {

    private static final Logger logger = LoggerFactory.getLogger(UserService.class);

    @Autowired(required = false)
    private UserDao dao;

    @Override
    public UserDao getDao() {
        return dao;
    }

    public Boolean getUserByPhoneNumber(String phoneNumber) {
        Boolean flag = false;
        User user = dao.getUserByPhoneNumber(phoneNumber);
        if (user != null) {
            flag = true;
        }
        return flag;
    }

    public User getByPhoneNumber(String phoneNumber) {

        return dao.getUserByPhoneNumber(phoneNumber);
    }

    /**
     * update password
     **/
    public boolean updatePassword(User user) {
        List<String> encryptList = encryptList(user.getPlainPassword());
        Asserts.notNull(encryptList, "encrypt list must not be null.");
        user.setEncryptkey(encryptList.get(0));
        user.setPassword(encryptList.get(1));
        UserPage userPage = new UserPage();
        user.setTemphone(user.getTemphone());
        User findUser = dao.queryByList(userPage).get(0);
        User updateUser = new User();
        updateUser.setPassword(encryptList.get(1));
        updateUser.setEncryptkey(encryptList.get(0));
        updateUser.setTemphone(user.getTemphone());
        int result = dao.updateByPrimaryKeySelective(updateUser);
        if (result != 0) {
            return true;
        } else {
            return false;
        }
    }


    /**
     * user login or register
     **/
    public boolean loginOrRegister(String temphone, String plainPassword) {
        UserPage userPage = new UserPage();
        userPage.setTemphone(temphone);
        List<User> users = dao.queryByList(userPage);
        if (CollectionUtils.isEmpty(users)) {
            List<String> encryptList = encryptList(plainPassword);
            Asserts.notNull(encryptList, "encrypt list must not be null.");
            String id = UUID.randomUUID();
            User insertUser = new User(id, temphone, encryptList.get(0), encryptList.get(1));
            logger.info("insertUser: {}", insertUser);
            int result = dao.insertSelective(insertUser);
            if (result != 0) {
                return true;
            } else {
                return false;
            }
        } else {
            User user = users.get(0);
            String decryptPassword = SecurityUtils.decrypt(user.getEncryptkey(), user.getPassword());
            Asserts.notNull(plainPassword, "plainPassword must not be null.");
            if (plainPassword.equals(decryptPassword)) {
                return true;
            } else {
                return false;
            }
        }
    }


    /**
     * user password encrypt list,list.get(0) is encryptKey,list.get(0) is encrypt password
     **/
    private List<String> encryptList(String plainPassword) {
        List<String> encryptList = new ArrayList<>(2);
        String encryptKey = UUID.randomUUID();
        encryptList.add(encryptKey);
        String encryptPassword = SecurityUtils.encrypt(plainPassword, encryptKey);
        encryptList.add(encryptPassword);
        logger.info("encryptList: {},encryptPassword: {}", encryptList, encryptPassword);
        return encryptList;
    }


    public List<Map<String,Object>> queryIpByUser(String phone){
        return dao.queryIpByUser(phone);
    }

    public void updateClientIdByPhoneNumber(String clientId, String phoneNumber) {
        dao.updateClientIdByPhoneNumber(clientId, phoneNumber);
    }

    public List<Map<String,Object>> queryClientByIp(@Param("ip") String ip){
        return dao.queryClientByIp(ip);
    }
}
