package com.adc.da.znks.util;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTPReply;

import java.io.*;
import java.util.Arrays;
import java.util.List;

/**
 * @ClassName FtpUtil ftp工具类
 * @Descroption TODO
 * @Author 王夏晖
 * @Date 2018/8/29 0029 15:38
 **/
public class FtpUtil {
    //FTP客户端
    private static  FTPClient ftp;
    //FTP服务器hostname
    private String host;
    //FTP服务器端口
    private int port;
    //FTP登录账号
    private String username;
    //FTP登录密码
    private String password;

    public FtpUtil(String host,int port,String username,String password){
        this.host = host;
        this.port = port;
        this.username = username;
        this.password = password;
    }

    /**
     * ftp用户登录
     * @return
     */
    public  boolean ftpLogin(){
        boolean flag = false;
        ftp = new FTPClient();
        try{
            ftp.connect(host,port);
            ftp.login(username,password);
            flag = true;
        }catch(Exception e){
            e.printStackTrace();
            ftp = null;
        }
        return flag;
    }

    /**
     * 获取目录下文件名称列表
     * @param ftpDirectory 文件路径
     * @return
     * @throws Exception
     */
    public List<String> getFileNameList(String ftpDirectory) throws Exception{
        if(ftpLogin()){
            String[] names = ftp.listNames(ftpDirectory);
            return Arrays.asList(names);
        }
       return null;
    }
    /**
     * Description: 向FTP服务器上传文件
     * @param basePath FTP服务器基础目录
     * @param filePath FTP服务器文件存放路径。例如分日期存放。文件的路径为basePath+filePath
     * @param filename 上传到FTP服务器上的文件名
     * @param input 输入流
     * @return 成功返回true，否则返回false
     */
    public  boolean ftpUpload(String basePath,
            String filePath, String filename, InputStream input,String is_Linux,String ftp_transport) {
        boolean result = false;
        boolean isLinux = false;
        boolean ftpTransport = true;
        if ("1".equals(is_Linux)){
            isLinux = true;
        }
        if ("0".equals(ftp_transport)){
            ftpTransport = false;
        }
       // FTPClient ftp = new FTPClient();
        try {
            int reply;
            /*ftp.connect(host, port);// 连接FTP服务器
            // 如果采用默认端口，可以使用ftp.connect(host)的方式直接连接FTP服务器
            boolean bool = ftp.login(username, password);// 登录
            System.out.println("ftp---denglu---"+bool);*/
            if(!ftpLogin()){
                System.out.println("ftp---denglu---失败");
                return false;
            }
            System.out.println("ftp---denglu---成功");
            //////////////////
            //编码格式的设置
            if (isLinux){
                //linux服务器
                filename = new String(filename.getBytes("utf-8"),"ISO8859-1");
            }else {
                //window服务器
                filename = new String(filename.getBytes("GBK"),"ISO8859-1");

            }
            //ftp传输模式
            if (ftpTransport){
                //设置为主动传输模式
                ftp.enterLocalPassiveMode();
            }else {
                //设置为被动传输模式
                ftp.enterLocalActiveMode();
            }
            System.out.println("-----------ip:"+host);
            System.out.println("-----------端口:"+port);
            System.out.println("-----------用户名:"+username);
            System.out.println("-----------密码:"+password);
            /////////////////
            reply = ftp.getReplyCode();
            if (!FTPReply.isPositiveCompletion(reply)) {
                System.out.println("-------51----:");
                ftp.disconnect();
                return result;
            }
            System.out.println("-------55----:");
            //切换到上传目录
            System.out.println("shangcuan----mulu-----"+basePath+filePath);
            if (!ftp.changeWorkingDirectory(basePath+filePath)) {
                System.out.println("-------57----:");
                //如果目录不存在创建目录
                String[] dirs = filePath.split("/");
                String tempPath = basePath;
                for (String dir : dirs) {
                    if (null == dir || "".equals(dir)) continue;
                    tempPath += "/" + dir;
                    if (!ftp.changeWorkingDirectory(tempPath)) {
                        if (!ftp.makeDirectory(tempPath)) {
                            return result;
                        } else {
                            ftp.changeWorkingDirectory(tempPath);
                        }
                    }
                }
            }
            //设置上传文件的类型为二进制类型
            ftp.setFileType(FTP.BINARY_FILE_TYPE);
            //上传文件
            System.out.println("-------77----:");
            if (!ftp.storeFile(filename, input)) {
                System.out.println("-------79----:");
                return result;
            }
            System.out.println("-------87----:");
            input.close();
            ftp.logout();
            result = true;
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (ftp.isConnected()) {
                try {
                    ftp.disconnect();
                } catch (IOException ioe) {
                    ioe.printStackTrace();
                }
            }
        }
        return result;
    }

    /**
     * 下载链接配置
     * @param f
     * @param localBaseDir 本地目录
     * @param remoteBaseDir 远程目录
     * @throws Exception
     */
    public  boolean startDown(String localBaseDir,String remoteBaseDir ) throws Exception{
        if (ftpLogin()) {
            try {
                FTPFile[] files = null;
                boolean changedir = ftp.changeWorkingDirectory(remoteBaseDir);
                if (changedir) {
                    ftp.setControlEncoding("GBK");
                    files = ftp.listFiles();
                    for (int i = 0; i < files.length; i++) {
                        try{
                            downloadFile(files[i], localBaseDir, remoteBaseDir);
                        }catch(Exception e){
                            System.out.println("下载失败");
                        }
                    }
                }
            } catch (Exception e) {
                System.out.println("下载失败");
            }
        }else{
            System.out.println("连接失败");
        }
        return false;
    }


    /**
     *
     * 下载FTP文件
     * 当你需要下载FTP文件的时候，调用此方法
     * 根据获取的文件名，本地地址，远程地址进行下载
     *
     * @param ftpFile
     * @param relativeLocalPath
     * @param relativeRemotePath
     */
    private  static void downloadFile(FTPFile ftpFile, String relativeLocalPath,String relativeRemotePath) {
        if (ftpFile.isFile()) {
            if (ftpFile.getName().indexOf("?") == -1) {
                OutputStream outputStream = null;
                try {
                    File entryDir = new File(relativeLocalPath);
                    //如果文件夹路径不存在，则创建文件夹
                    if (!entryDir.exists() || !entryDir.isDirectory())
                    {
                        entryDir.mkdirs();
                    }
                    File locaFile= new File(relativeLocalPath+ ftpFile.getName());
                    //判断文件是否存在，存在则返回
                    if(locaFile.exists()){
                        return;
                    }else{
                        outputStream = new FileOutputStream(relativeLocalPath+ ftpFile.getName());
                        ftp.retrieveFile(ftpFile.getName(), outputStream);
                        outputStream.flush();
                        outputStream.close();
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    try {
                        if (outputStream != null){
                            outputStream.close();
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        } else {
            String newlocalRelatePath = relativeLocalPath + ftpFile.getName();
            String newRemote = new String(relativeRemotePath+ ftpFile.getName().toString());
            File fl = new File(newlocalRelatePath);
            if (!fl.exists()) {
                fl.mkdirs();
            }
            try {
                newlocalRelatePath = newlocalRelatePath + '/';
                newRemote = newRemote + "/";
                String currentWorkDir = ftpFile.getName().toString();
                boolean changedir = ftp.changeWorkingDirectory(currentWorkDir);
                if (changedir) {
                    FTPFile[] files = null;
                    files = ftp.listFiles();
                    for (int i = 0; i < files.length; i++) {
                        downloadFile(files[i], newlocalRelatePath, newRemote);
                    }
                }
                if (changedir){
                    ftp.changeToParentDirectory();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public static void main(String args[]) throws Exception {
        FtpUtil ftp = new FtpUtil("192.168.0.102",21,"ftpuser","123456");
        System.out.println(ftp.getFileNameList("/home/json/acc01"));
        ftp.startDown("E:/haha/","/home/json/acc01/");
    }
}
