package com.adc.da.znks.util.netty;

import com.adc.da.znks.service.GatewayService;
import com.adc.da.znks.util.SocketUtils;
import com.adc.da.znks.util.SystemValue;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.bytes.ByteArrayDecoder;
import io.netty.handler.codec.bytes.ByteArrayEncoder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;

public class NettyServer {
	private final int port;

	public NettyServer(int port) {
		this.port = port;
	}

	public void start() throws Exception {
		EventLoopGroup group = new NioEventLoopGroup();
		try {
			ServerBootstrap sb = new ServerBootstrap();
			sb.group(group) // 绑定线程池
					.channel(NioServerSocketChannel.class) // 指定使用的channel
					.localAddress(this.port)// 绑定监听端口
					.childHandler(new ChannelInitializer<SocketChannel>() { // 绑定客户端连接时候触发操作

						@Override
						protected void initChannel(SocketChannel ch) throws Exception {
							System.out.println("connected...; Client:" + ch.remoteAddress().getHostString());
							//發送請求指令
							if( SystemValue.ipMacMap.get(ch.remoteAddress().getHostString()) == null){
								connectGateway(ch.remoteAddress().getHostString(),8899);
							}
							ch.pipeline().addLast(new ByteArrayEncoder());
							ch.pipeline().addLast(new ByteArrayDecoder());
							ch.pipeline().addLast(new ServerHandler()); // 客户端触发操作



						}
					});
			ChannelFuture cf = sb.bind().sync(); // 服务器异步创建绑定
			System.out.println(NettyServer.class + " started and listen on " + cf.channel().localAddress());
			cf.channel().closeFuture().sync(); // 关闭服务器通道
		} finally {
			group.shutdownGracefully().sync(); // 释放线程池资源
		}
	}

	public void connectGateway(String host,int port){
		byte[] request = new byte[23];
		request[0] = 0x4d;
		request[1] = 0x41;
		request[2] = 0x43;
		request[3] = 0x5F;
		request[4] = 0x52;
		request[5] = 0x45;
		request[6] = 0x51;
		request[7] = 0x55;
		request[8] = 0x45;
		request[9] = 0x53;
		request[10] = 0x54;
		SocketUtils.connectServerWithTCPSocket(request,1,host,port);
	}
}