package com.adc.da.znks.util.netty;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageDecoder;

import java.util.List;

/**
 * 57所netty 服务端接受数据 拆包
 *
 * @ClassName:RequestDecoder
 * @author: DuYunbao
 * @date: 2018/3/22 14:27
 */
public class RequestDecoder extends ByteToMessageDecoder {


    private byte[] headAndLength = new byte[24];

    @Override
    protected void decode(ChannelHandlerContext ctx, ByteBuf buffer, List<Object> out) throws Exception {
        //可读长度必须大于 同步位置到包长度属性 的长度
        if (buffer.readableBytes() >= headAndLength.length) {
            //防止socket字节流攻击
            /*if (buffer.readableBytes() > 2048) {
                buffer.skipBytes(buffer.readableBytes());
            }*/

            //记录包头开始的index
            int beginReader;
            byte[] synchroSign;
            while (true) {
                beginReader = buffer.readerIndex();
                buffer.markReaderIndex();
                synchroSign = new byte[4];
                buffer.readBytes(synchroSign);
                //开始判断当前读取数据是否是包头数据
                if (synchroSign[0] == -2 && synchroSign[1] == -17 && synchroSign[2] == -97 && synchroSign[3] == -7) {
                    break;
                }
                //长度又变得不满足
                if (buffer.readableBytes() < headAndLength.length) {
                    return;
                }
            }
            //版本号
            byte[] versionNumber = new byte[2];
            buffer.readBytes(versionNumber);

            //设备标识
            byte[] deviceSign = new byte[14];
            buffer.readBytes(deviceSign);

            //包长度
            byte[] packageData = new byte[4];
            buffer.readBytes(packageData);
            int dataLength = ByteToInt(packageData);

            //判断请求数据包数据是否到齐
            if (buffer.readableBytes() < dataLength - 24) {
                //还原读指针
                buffer.readerIndex(beginReader);
                return;
            }

            //读取data数据
            byte[] data = new byte[dataLength - 24];
            buffer.readBytes(data);

            //继续往下传递
            byte[] bytes = new byte[dataLength];
            System.arraycopy(synchroSign, 0, bytes, 0, synchroSign.length);
            int index1 = synchroSign.length;
            System.arraycopy(versionNumber, 0, bytes, index1, versionNumber.length);
            int index2 = versionNumber.length + synchroSign.length;
            System.arraycopy(deviceSign, 0, bytes, index2, deviceSign.length);
            int index3 = versionNumber.length + synchroSign.length + deviceSign.length;
            System.arraycopy(packageData, 0, bytes, index3, packageData.length);
            int index4 = versionNumber.length + synchroSign.length + deviceSign.length + packageData.length;
            System.arraycopy(data, 0, bytes, index4, data.length);
            out.add(bytes);
        }
    }

    public int ByteToInt(byte[] bArr) {
        if(bArr.length!=4){
            return -1;
        }
        return (int) ((((bArr[3] & 0xff) << 24)
                | ((bArr[2] & 0xff) << 16)
                | ((bArr[1] & 0xff) << 8)
                | ((bArr[0] & 0xff) << 0)));
    }

}
