package com.adc.da.znks.util.netty;

import com.adc.da.znks.entity.Gateway;
import com.adc.da.znks.entity.LockRecord;
import com.adc.da.znks.service.GatewayService;
import com.adc.da.znks.service.LockRecordService;
import com.adc.da.znks.service.UserService;
import com.adc.da.znks.util.AppPushUtils;
import com.adc.da.znks.util.SocketUtils;
import com.adc.da.znks.util.SystemValue;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import oracle.jdbc.proxy.annotation.Post;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import sun.java2d.pipe.SpanShapeRenderer;

import javax.annotation.PostConstruct;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 接收socket请求
 * @author David
 */
@Component
public class ServerHandler extends ChannelInboundHandlerAdapter {

	@Autowired
	private GatewayService gatewayService;
	@Autowired
	private LockRecordService lockRecordService;
	@Autowired
	private UserService userService;

	public static ServerHandler serverHandler;

	@PostConstruct
	public void init(){
		serverHandler = this;
		serverHandler.gatewayService = this.gatewayService;
		serverHandler.lockRecordService = this.lockRecordService;
		serverHandler.userService = this.userService;
	}

	@Override
	public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
		System.out.println("server channelRead...; received:" + msg);
		String remoteAddress = ctx.channel().remoteAddress().toString();
		String host = remoteAddress.substring(1,remoteAddress.lastIndexOf(":"));
		System.out.println("网关IP地址：" + host);
		byte[] result = (byte[])msg;
		System.out.println(SocketUtils.bytesToHex(result,0,result.length));
		// 在此处需要分别解析
		// 1.上报MAC地址信息 6位mac地址+10位的（MAC-UPLOAD）
		try{
			int len = SocketUtils.bytesToHex(result,0,result.length).split(" ").length;
			if(len == 16){
				String mac = SocketUtils.bytesToHex(result,0,6);
				Gateway gateway = new Gateway();
				gateway.setIp(host);
				gateway.setMacAddress(mac);
				serverHandler.gatewayService.updateIpByMac(gateway);
				SystemValue.ipMacMap.put(host, mac);
			}else{
				String title = "";
				String message = "";
				String clientId = "";
				List<Map<String,Object>> list = serverHandler.userService.queryClientByIp(host);
				// 2.开关门信息（ 0x54 0x59 0x54 0x01 0x01 0x00 0x01 0x01 0x05 0x00 0x01）
				// 前6位不用管，第七位是设备编号，第八位是功能码0x05代表开锁，0x06代表关锁，最后两位代表用户ID，1-300

				// 3.安防信息，需要根据安防信息做不同的处理，安防信息有 0x60(非法开门警报)，0x61（非法尝试开锁警报），0x80(远程开锁触发)，0x28(电池低电量警报)
				byte device = result[13];
				byte user = 0;
				String device_id = String.valueOf(device);
				byte instruct = 0;
				if(result.length == 17){//开关门信息，6位mac + 11位指令
					instruct = result[14];
					user = result[16];
				}else{//开关门信息，6位mac + 8位指令
					instruct = result[9];
				}
				int user_id = Integer.parseInt(String.valueOf(user));
				String lockType = getLockType(instruct);
				title = getTitle(instruct);
				message = getMessage(instruct);
				LockRecord lockRecord = new LockRecord();
				lockRecord.setTime(new Date());
				lockRecord.setUser(user_id);
				lockRecord.setDeviceId(device_id);
				lockRecord.setLockType(lockType);
				System.out.println(lockType);
				serverHandler.lockRecordService.insertLockRecord(lockRecord);
				if(list != null){
					clientId = list.get(0).get("client_id")!=null?list.get(0).get("client_id").toString():"";
					if(!"".equals(clientId)){
						AppPushUtils.push(title,message,clientId);
					}
				}else{
					System.out.println("clientId未获取，消息发送失败");
				}


			}
		}catch(Exception e){
			System.out.println("数据存储失败");
			e.printStackTrace();
		}


		//ctx.write(msg);
	}

	@Override
	public void channelReadComplete(ChannelHandlerContext ctx) throws Exception {
		System.out.println("server channelReadComplete..");
		// 第一种方法：写一个空的buf，并刷新写出区域。完成后关闭sock channel连接。
		ctx.writeAndFlush(Unpooled.EMPTY_BUFFER).addListener(ChannelFutureListener.CLOSE);
	}

	@Override
	public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
		System.out.println("server occur exception:" + cause.getMessage());
		cause.printStackTrace();
		// 关闭发生异常的连接
		ctx.close();
	}

	public String convertByteBufToString(ByteBuf buf) {
		String str;
		if(buf.hasArray()) {
			// 处理堆缓冲区
			str = new String(buf.array(), buf.arrayOffset() + buf.readerIndex(), buf.readableBytes());
		} else { // 处理直接缓冲区以及复合缓冲区
			byte[] bytes = new byte[buf.readableBytes()];
			buf.getBytes(buf.readerIndex(), bytes);
			str = new String(bytes, 0, buf.readableBytes());
		}
		return str;
	}

	//判断指令是不是mac地址
	public boolean ifMac(byte[] b){
		if(b[6] == 0x54 && b[7] == 0x59 && b[8] == 0x54 && b[9] == 0x01 ){
			return false;
		}
		return true;
	}

	public String getLockType(byte b){
		if(b == 0x05){return "开锁";}
		else if(b == 0x06){return "关锁";}
		else if(b == 0x60){return "非法开门警报";}
		else if(b == 0x61){return "非法尝试开锁警报";}
		else if(b == 0x80){return "远程开锁触发";}
		else if(b == 0x28){return "电池低电量警报";}
		return "";
	}

	public String getTitle(byte b){
		String title = "";
		if(b == 0x05){title = "开锁";}
		else if(b == 0x06){title = "关锁";}
		else if(b == 0x60){title = "非法开门警报";}
		else if(b == 0x61){title = "非法尝试开锁警报";}
		else if(b == 0x80){title = "远程开锁触发";}
		else if(b == 0x28){title = "电池低电量警报";}
		return title;
	}

	public String getMessage(byte b){
		String msg = "";
		if(b == 0x05){msg = "开锁";}
		else if(b == 0x06){msg = "关锁";}
		else if(b == 0x60){msg = "非法开门警报";}
		else if(b == 0x61){msg = "非法尝试开锁警报";}
		else if(b == 0x80){msg = "远程开锁触发";}
		else if(b == 0x28){msg = "电池低电量警报";}
		SimpleDateFormat simple = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		return simple.format(new Date()) + " 触发 " + msg;
	}

	public static int bytesToInt(byte[] src, int offset) {
		int value;
		value = (int) ((src[offset] & 0xFF)
				| ((src[offset+1] & 0xFF)<<8)
				| ((src[offset+2] & 0xFF)<<16)
				| ((src[offset+3] & 0xFF)<<24));
		return value;
	}

}